
#include <SPI.h>          // библиотека для работы с шиной SPI
#include "nRF24L01.h"     // библиотека радиомодуля
#include "RF24.h"         // ещё библиотека радиомодуля
#include <Servo.h>

Servo myservo1;
Servo myservo2;
Servo myservo3;
Servo myservo4;
Servo myservo5;

RF24 radio(9, 10); // "создать" модуль на пинах 9 и 10 Для Уно
//RF24 radio(9,53); // для Меги

byte address[][6] = {"1Node", "2Node", "3Node", "4Node", "5Node", "6Node"}; //возможные номера труб

byte counter;

int glove[8]; // массив данных для отправки
int manipulator[5]; // входящий массив от манипулятора (обратная тактильная связь)
int buf[8][5]; //массив для очистки шумов


void setup() {
  Serial.begin(9600);         // открываем порт для связи с ПК

  radio.begin();              // активировать модуль
  radio.setAutoAck(1);        // режим подтвержденzия приёма, 1 вкл 0 выкл
  radio.setRetries(0, 15);    // (время между попыткой достучаться, число попыток)
  radio.enableAckPayload();   // разрешить отсылку данных в ответ на входящий сигнал
  radio.setPayloadSize(32);   // размер пакета, в байтах

  radio.openWritingPipe(address[0]);// мы - труба 0, открываем канал для передачи данных
  radio.openReadingPipe(0, address[1]);  // + открываем чтение сигнала
  radio.setChannel(0x71);             // выбираем канал (в котором нет шумов!)

  radio.setPALevel (RF24_PA_MAX);   // уровень мощности передатчика. На выбор RF24_PA_MIN, RF24_PA_LOW, RF24_PA_HIGH, RF24_PA_MAX
  radio.setDataRate (RF24_2MBPS); // скорость обмена. На выбор RF24_2MBPS, RF24_1MBPS, RF24_250KBPS
  //должна быть одинакова на приёмнике и передатчике!
  //при самой низкой скорости имеем самую высокую чувствительность и дальность!!

  radio.powerUp();        // начать работу
  radio.startListening(); // начинаем слушать эфир

  Serial.println("Started");  // оповещаем оператара, что живы

  // прописываем пины, к которым подключены сервоприводы
  myservo1.attach(4);
  myservo2.attach(5);
  myservo3.attach(6);
  myservo4.attach(7);
  myservo5.attach(3);

  //  прописываем входы сигналов от потенцеометров с пальцев
  pinMode (A3, INPUT);
  pinMode (A4, INPUT);
  pinMode (A5, INPUT);
  pinMode (A6, INPUT);
  pinMode (A7, INPUT);
  // и с кистевого сустава
  pinMode (A0, INPUT);  // вращение предплечия вокруг своей оси
  pinMode (A1, INPUT);  // вращение кисти влево-вправо
  pinMode (A2, INPUT);  // и вверх-вниз

  // заполняем массив нулями
  int i = 0;
  while (i < 8) {
    int b = 0;
    while (b < 5) {
      buf[i][b] = 0;
      b ++;
    }
    i ++;
  }
}

int pot[8]; // массив переменных для чтения аналогово сигнала потенцеометров
int signal[5]; // + массив для полученя обратной связи
int servo_buf[5];// + массив буфера для остановки двигателей

void loop() {
  byte pipeNo, gotByte; // прописываем, что хотим слушать любую трубу

  //  читаем сигнал потенцеометров
  pot[0] = map(analogRead(A3), 0, 1023, 300, 0);
  pot[1] = map(analogRead(A4), 0, 1023, 300, 0);
  pot[2] = map(analogRead(A5), 0, 1023, 300, 0);
  pot[3] = map(analogRead(A7), 0, 1023, 300, 0);
  pot[4] = map(analogRead(A6), 0, 1023, 300, 0);

  pot[5] = map(analogRead(A0), 1023, 700, 180, 0); // вращение предплечия вокруг оси
  pot[6] = map(analogRead(A1), 1023, 0, 300, 0); // кисть влево-вправо
  pot[7] = map(analogRead(A2), 1023, 0, 300, 0); // кисть вверх-вниз

  // обновляем буфер значений для фильтрации шумов
  int i = 0;
  while (i < 8){
    int b = 0;
    while (b < 4){
      buf[i][b] = buf[i][b + 1];
      b ++;
    }
    buf[i][4] = pot[i]; 
    i++;
  }

  i = 0;
  while (i < 8){
    
    // фильтруем средним арифметическим
    int b = 0;
    int sum = 0;
    while (b < 5){sum += buf[i][b]; b ++;}
    pot [i] = sum / 5;
    // защита сервоприводов от недопонимания
    if (pot[i] > 179){pot[i] = 179;}
    if (pot[i] < 1){pot[i] = 1;}

    // заполняем массив для отправки
    glove[i] = pot[i];

    i ++;
  }

  // инвертируем нужные сервы (из-за конструкции)
  pot[4] = map(pot[4], 0, 180, 180, 0);

  // корректируем обратную тактильную связь
  pot[0] += 15;
  pot[1] -= 13;
  pot[2] += 15;
  pot[3] -= 5;

  i = 0;
  while (i < 8){
    // защита сервоприводов от недопонимания
    if (pot[i] > 179){pot[i] = 180;}
    if (pot[i] < 1){pot[i] = 1;}
    i ++;
  }

  // при поступлении сигнала, блокируем сервопривод
  i = 0;
  while (i < 5){
    if (signal[i] > 10) {pot[i] = servo_buf[i];}
    i ++;
  }
  // радуем оператора, тем, что всё ещё живы
  Serial.print(190); Serial.print(" ");
  Serial.print(-10); Serial.print(" ");

  i = 0;
  while (i < 8){
    Serial.print(pot[i]); Serial.print(" ");
    i ++;
  }
  Serial.println(" ");

  // посылаем сигнал на сервоприводы
  myservo1.write(pot[0]);
  myservo2.write(pot[1]);
  myservo3.write(pot[2]);
  myservo4.write(pot[3]);
  myservo5.write(pot[4]);

  radio.stopListening(); // перестаём слушать эфир
  radio.write(&glove, sizeof(glove)); // отправляем данные
  radio.startListening(); // начинаем слушать эфир
  
  i = 0;
  while (i < 5){
    signal[i] = 0;
    i ++;
  }

  if (radio.available(&pipeNo)) {  // слушаем эфир со всех труб
    radio.read(&manipulator, sizeof(manipulator));  // чиатем входящий сигнал

    // заполняем переменные с помощью принятого массива
    i = 0;
    while (i < 5){
      signal[i] = manipulator[i];
      i ++;
    }

    // радуем оператора, тем, что всё ещё живы
    Serial.print("Принято: "); Serial.print("          ");
    i = 0;
    while (i < 5){
      Serial.print(signal[i]); Serial.print(" ");
      i ++;
    }
    Serial.println(" ");
  }

  // обновляем буфер если можно
  i = 0;
  while (i < 5){
    if (signal[i] > 10) {servo_buf[i] = pot[i] - 15;}
    i ++;
  }

  delay(50); // задержка для корректной работы сервоприводов
}
