#include <SPI.h>          // библиотека для работы с шиной SPI
#include "nRF24L01.h"     // библиотека радиомодуля
#include "RF24.h"         // ещё библиотека радиомодуля
#include <Servo.h>


RF24 radio(9, 10); // "создать" модуль на пинах 9 и 10

byte address[][6] = {"1Node", "2Node", "3Node", "4Node", "5Node", "6Node"}; //возможные номера труб

byte counter;

int glove[8]; // входящий массив от перчатки
int manipulator[5]; // массив для отправки (обратная тактильная связь)

// прописываем, какие сервы хотим использывать
Servo servo_1;
Servo servo_2;
Servo servo_3;
Servo servo_4;
Servo servo_5;

void setup() {
  Serial.begin(9600);         // открываем порт для связи с ПК

  radio.begin();              // активировать модуль
  radio.setAutoAck(1);        // режим подтверждения приёма, 1 вкл 0 выкл
  radio.setRetries(0, 15);    // (время между попыткой достучаться, число попыток)
  radio.enableAckPayload();   // разрешить отсылку данных в ответ на входящий сигнал
  radio.setPayloadSize(32);   // размер пакета, в байтах

  radio.openWritingPipe(address[1]);  // мы - труба 1, открываем канал для передачи данных
  radio.openReadingPipe(1, address[0]);  // + открываем чтение сигнала
  radio.setChannel(0x71);             // выбираем канал (в котором нет шумов!)

  radio.setPALevel (RF24_PA_MAX);   // уровень мощности передатчика. На выбор RF24_PA_MIN, RF24_PA_LOW, RF24_PA_HIGH, RF24_PA_MAX
  radio.setDataRate (RF24_2MBPS); // скорость обмена. На выбор RF24_2MBPS, RF24_1MBPS, RF24_250KBPS
  //должна быть одинакова на приёмнике и передатчике!
  //при самой низкой скорости имеем самую высокую чувствительность и дальность!!

  radio.powerUp();        // начать работу
  radio.startListening();  // начинаем слушать эфир

  Serial.println("Started");  // оповещаем оператара, что живы

  // прописываем пины, к которым подключены сервоприводы
  servo_1.attach(2);  //вращение предплечия вокруг оси
  servo_2.attach(A0); //кисть вверх-вниз 
  servo_3.attach(3); //кисть влево-вправо
  servo_4.attach(4); // 4 пальца 
  servo_5.attach(5); //большой палец 

  // прописываем пины концевиков
  pinMode(7, INPUT);
  pinMode(8, INPUT);

}

// переменные для чтения принятого сигнала 
//int signal[8];
int buf[5]; // массив максимальных сигналов для обратной связи

// переменные для считывания сигнала концевиков (обратная связь)
int but[2];

void loop() {
  byte pipeNo, gotByte; // прописываем, что хотим слушать любую трубу

  // чипаем сигналы с концевиков и заполняем переменные
  but[0] = digitalRead(8);
  but[1] = digitalRead(7);

  if (radio.available(&pipeNo)) {        // слушаем эфир со всех труб
    radio.read(&glove, sizeof(glove));  // чиатем входящий сигнал

    glove[0] = map(glove[0], 0, 60, 0, 90);
    glove[6] = map(glove[6], 0, 181, 180, 0);
    glove[7] = map(glove[7], 0, 180, 0, 80);

    // радуем оператора, тем, что всё ещё живы
    int i = 0;
    while (i < 8){
      Serial.print(glove[i]); Serial.print(" ");
      i ++;
    }
    Serial.print("             "); 
    Serial.print(but[0]); Serial.print(" ");
    Serial.print(but[1]); Serial.println(" ");

    // проверяем: если концевик нажат, то не даём сервоприводу двигаться дальше
    if (but[0] == 1){
      buf[0] = glove[0];
      if (glove[0] > buf[0]) {glove[0] = buf[0];}
      }
    if (but[1] == 1){
      i = 1;
      while (i < 5){
        buf[i] = glove[i];
        if (glove[i] > buf[i]) {glove[i] = buf[i];}
        i ++;
      }
    }

    // посылаем, принятый сигнал на сервоприводы
    int sp_signal = (glove[1] + glove[2] + glove[3] + glove[4]) / 4;
    sp_signal = map (sp_signal, 0, 140, 0, 90);
    servo_1.write(glove[5]);
    servo_2.write(glove[7]);
    servo_3.write(glove[6]);
    servo_4.write(sp_signal);
    servo_5.write(glove[0]);
  }

  if (but[0] == 1 or but[1] == 1){  // проверяем нажат ли концевик
    // обнуляем массив
    manipulator[0] = 0;
    manipulator[1] = 0;
    
    // заполняем массив при надобности
    if (but[0] == 1){manipulator[0] = 100;}
    if (but[1] == 1){
      int i = 1;
      while (i < 5){
        manipulator[i] = 100;
        i ++;
      }
    }

    radio.stopListening(); // перестаём слушать эфир
    radio.write(&manipulator, sizeof(manipulator)); // отправляем данные
    radio.startListening(); // начинаем слушать эфир
  }

  delay(20); // задержка для радио
}
